package com.caimei365.order.mapper;

import com.caimei365.order.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/9
 */
@Mapper
public interface BaseMapper {
    /**
     * 获取机构用户Id
     * @param clubId 机构Id
     */
    Integer getUserIdByClubId(Integer clubId);
    /**
     * 根据用户Id查询用户身份
     */
    Integer getIdentityByUserId(Integer userId);
    /**
     * 根据商品ID查询阶梯价列表
     */
    List<LadderPriceVo> getLadderPriceList(Integer productId);
    /**
     * 根据商品ID和用户ID 查询复购价
     */
    Double getRepurchasePrice(@Param("productId") Integer productId, @Param("userId") Integer userId);
    /**
     * 供应商促销优惠活动
     * @param shopId 供应商Id
     */
    PromotionsVo getPromotionByShopId(Integer shopId);
    /**
     * 商品促销优惠活动
     * @param productId 商品Id
     */
    PromotionsVo getPromotionByProductId(Integer productId);
    /**
     * 促销优惠活动赠品列表
     * @param promotionsId 促销Id
     */
    List<CartItemVo> getPromotionGifts(Integer promotionsId);
    /**
     * 获取赠品供应商
     * @param productId 商品Id
     */
    CartShopVo getPromotionGiftsShop(Integer productId);
    /**
     * 发票信息
     * @param userId 用户Id
     */
    InvoiceVo getUserInvoice(Integer userId);
    /**
     * 可用余额
     * @param userId 用户Id
     */
    Double getUserAbleUserMoney(Integer userId);
    /**
     * 可用采美豆
     * @param userId 用户Id
     */
    Integer getUserBeans(Integer userId);
    /**
     * 获取商品运费 （0包邮 1到付 2默认（遵循运费规则））
     * @param productIds 商品Id列表
     */
    List<ProductPostageVo> getPostageFlagList(@Param("productIds") List<String> productIds);
    /**
     * 统计用户订单数
     * @param userId 用户Id
     */
    Integer countUserOrder(Integer userId);
    /**
     * 获取根据地区Id获取省市地址信息
     * @param townId 地区Id
     */
    AddressVo getProvinceIdAndCityId(Integer townId);
}
