package com.caimei365.order.mapper;

import com.caimei365.order.model.dto.SellerCartDto;
import com.caimei365.order.model.po.SellerCartPo;
import com.caimei365.order.model.vo.CartItemVo;
import com.caimei365.order.model.vo.CartShopVo;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
@Mapper
public interface CartSellerMapper {
    /**
     * 协销购物车供应商列表
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     */
    List<CartShopVo> getSellerCartShops(Integer serviceProviderId, Integer clubId);
    /**
     * 协销购物车供应商下商品列表
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     * @param shopId             供应商Id
     */
    List<CartItemVo> getSellerCartProducts(Integer serviceProviderId, Integer clubId, Integer shopId);

    /**
     * 获取协销购物车商品种类
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     */
    Integer getSellerCartCount(Integer serviceProviderId, Integer clubId);
    /**
     * 获取协销购物车
     * @param sellerCartDto clubId,productId,serviceProviderId
     */
    SellerCartPo getSellerCart(SellerCartDto sellerCartDto);
    /**
     * 根据Id获取协销购物车
     * @param id 购物车id
     */
    SellerCartPo getSellerCartById(Integer id);
    /**
     * 更新购物车
     * @param sellerCartPo SellerCartPo
     */
    void updateSellerCart(SellerCartPo sellerCartPo);
    /**
     * 添加购物车
     * @param sellerCartPo SellerCartPo
     */
    void insertSellerCart(SellerCartPo sellerCartPo);
    /**
     * 删除购物车
     * @param serviceProviderId  协销Id
     * @param cartIds 购物车Id集合
     */
    void deleteSellerCartByIds(Integer serviceProviderId, List<Integer> cartIds);
}
