package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.CartSellerMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.SellerCartDto;
import com.caimei365.order.model.po.SellerCartPo;
import com.caimei365.order.model.vo.CartItemVo;
import com.caimei365.order.model.vo.PromotionPriceVo;
import com.caimei365.order.model.vo.PromotionsVo;
import com.caimei365.order.model.vo.CartShopVo;
import com.caimei365.order.service.CartSellerService;
import com.caimei365.order.utils.MathUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
@Slf4j
@Service
public class CartSellerServiceImpl implements CartSellerService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private CartSellerMapper cartSellerMapper;
    @Resource
    private ProductService productService;

    /**
     * 协销购物车列表数据
     *
     * @param serviceProviderId  协销Id
     * @param clubId             机构Id
     * @param againBuyProductIds 再来一单商品Ids
     * @param pageNum            页码
     * @param pageSize           每页数量
     */
    @Override
    public ResponseJson<Map<String, Object>> getSellerCartList(Integer serviceProviderId, Integer clubId, String againBuyProductIds, int pageNum, int pageSize) {
        List<Integer> againBuyIdList = Lists.newArrayList();
        if (againBuyProductIds.contains(",")) {
            String[] againBuyArr = againBuyProductIds.split(",");
            for (String id : againBuyArr) {
                againBuyIdList.add(Integer.parseInt(id));
            }
        } else {
            againBuyIdList.add(Integer.parseInt(againBuyProductIds));
        }
        // 失效商品列表（总）
        List<CartItemVo> invalidList = new ArrayList<>();
        // 促销活动（总）
        List<PromotionsVo> totalPromotions = new ArrayList<>();
        List<Integer> promotionsIds = new ArrayList<>();
        // 获取机构用户Id
        Integer clubUserId = baseMapper.getUserIdByClubId(clubId);
        // 开始分页请求供应商数据
        PageHelper.startPage(pageNum, pageSize);
        // 协销购物车供应商列表
        List<CartShopVo> shopInfoList = cartSellerMapper.getSellerCartShops(serviceProviderId, clubId);
        if (null != shopInfoList && shopInfoList.size()>0) {
            // 遍历供应商列表
            shopInfoList.forEach(shop -> {
                // 该供应商下商品种类
                AtomicInteger shopKindCount = new AtomicInteger(0);
                // 该供应商总价
                AtomicDouble shopPrice = new AtomicDouble(0);
                // 该供应商满减金额（供应商满减，单品满减）
                AtomicDouble shopReducedPrice = new AtomicDouble(0);
                // 供应商促销优惠活动
                PromotionsVo shopPromotion = baseMapper.getPromotionByShopId(shop.getShopId());
                // 供应商下商品列表
                List<CartItemVo> productList = cartSellerMapper.getSellerCartProducts(serviceProviderId, clubId, shop.getShopId());
                // 迭代器设置商品信息
                Iterator<CartItemVo> productIterator = productList.iterator();
                while (productIterator.hasNext()) {
                    CartItemVo cartItemVo = productIterator.next();
                    if (cartItemVo.getValidFlag() == 0) {
                        // 后台逻辑删除,已停售
                        cartItemVo.setStatus(1);
                        invalidList.add(cartItemVo);
                    } else if (cartItemVo.getValidFlag() == 10) {
                        // 已冻结，已丢失
                        cartItemVo.setStatus(2);
                        invalidList.add(cartItemVo);
                    } else {
                        // 设置商品有效
                        cartItemVo.setStatus(0);
                        // 设置商品图片及税费
                        boolean taxFlag = productService.setCartItemImgAndTax(cartItemVo);
                        // 获取商品促销信息
                        PromotionsVo promotions = null;
                        // 没有店铺促销时，商品促销才有效
                        if (null == shopPromotion) {
                            // 获取商品促销信息
                            promotions = baseMapper.getPromotionByProductId(cartItemVo.getProductId());
                            /*
                             * 设置商品促销优惠
                             */
                            if (null != promotions) {
                                // 当前促销活动的价格计算列表
                                List<PromotionPriceVo> promotionPriceList = productService.getPromotionProducts(promotions, cartItemVo, taxFlag);
                                // 更新到总促销列表
                                productService.updateTotalPromotions(totalPromotions, promotionsIds, promotions, promotionPriceList);
                                //单品满减-计算供应商总价/满减金额
                                if (promotions.getType() == 1 && promotions.getMode() == 2) {
                                    BigDecimal totalAmount = MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice());
                                    if (MathUtil.compare(totalAmount, promotions.getTouchPrice()) > -1) {
                                        // 如果满足促销条件，设置供应商价格-满减金额，满减总额 + 当前促销满减金额
                                        shopPrice.set(MathUtil.sub(shopPrice.get(), promotions.getReducedPrice()).doubleValue());
                                        shopReducedPrice.set(MathUtil.add(shopReducedPrice, promotions.getReducedPrice()).doubleValue());
                                    }
                                }
                                cartItemVo.setPromotions(promotions);
                            }
                        }
                        if (null != promotions || null != shopPromotion) {
                            // 商品处于活动状态
                            cartItemVo.setActStatus(1);
                            // 关闭阶梯价格，活动优先
                            cartItemVo.setLadderFlag(0);
                        } else {
                            if (cartItemVo.getLadderFlag() == 1) {
                                // 设置阶梯价
                                productService.setCartLadderPrices(cartItemVo, taxFlag);
                            } else {
                                // 复购价
                                Double repurchase = baseMapper.getRepurchasePrice(cartItemVo.getProductId(), clubUserId);
                                if (null != repurchase && repurchase > 0) {
                                    cartItemVo.setPrice(repurchase);
                                }
                            }
                        }
                        // 再来一单的商品默认选中
                        if (againBuyIdList.contains(cartItemVo.getProductId())) {
                            cartItemVo.setIsChecked(true);
                        }
                        // 该供应商下价格累加
                        shopPrice.set(MathUtil.add(shopPrice, MathUtil.mul(cartItemVo.getNumber(), cartItemVo.getPrice())).doubleValue());
                        // 该供应商下商品种类 +1
                        shopKindCount.incrementAndGet();
                    }
                }
                // 店铺促销
                if (null != shopPromotion) {
                    shop.setPromotions(shopPromotion);
                    if (!promotionsIds.contains(shopPromotion.getId())) {
                        promotionsIds.add(shopPromotion.getId());
                        // 店铺满赠
                        if (shopPromotion.getMode() == 3) {
                            // 获取赠品
                            List<CartItemVo> giftList = baseMapper.getPromotionGifts(shopPromotion.getId());
                            shopPromotion.setGiftList(giftList);
                        }
                        // 设置该优惠下的商品列表
                        List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
                        productList.forEach(item -> {
                            PromotionPriceVo promotionPrice = new PromotionPriceVo();
                            promotionPrice.setProductId(item.getProductId());
                            promotionPrice.setNumber(item.getNumber());
                            promotionPrice.setPrice(item.getPrice());
                            promotionPriceList.add(promotionPrice);
                        });
                        shopPromotion.setProductList(promotionPriceList);
                        // 添加到总促销
                        totalPromotions.add(shopPromotion);
                        // 店铺满减-计算供应商总价/满减金额
                        if (shopPromotion.getMode() == 2 && MathUtil.compare(shopPrice, shopPromotion.getTouchPrice()) > -1) {
                            // 该供应商总价 - 满减金额
                            shopPrice.set(MathUtil.sub(shopPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                            // 该供应商优惠总额 + 满减金额
                            shopReducedPrice.set(MathUtil.add(shopReducedPrice.get(), shopPromotion.getReducedPrice()).doubleValue());
                        }
                    }
                }
                // 供应商商品
                shop.setCartList(productList);
                // 供应商总价
                shop.setTotalPrice(shopPrice.get());
                // 供应商总优惠
                shop.setReducedPrice(shopReducedPrice.get());
                // 供应商划线价
                shop.setOriginalPrice(MathUtil.add(shopPrice.get(), shopReducedPrice.get()).doubleValue());
                // 供应商下商品种类
                shop.setCount(shopKindCount.get());
            });
            // 删除空数据
            shopInfoList.removeIf(shop -> (null == shop || shop.getCount() == 0));
        }
        PageInfo<CartShopVo> pageInfo = null;
        if (null != shopInfoList && shopInfoList.size()>0){
            pageInfo = new PageInfo(shopInfoList);
        }
        /*
         * 返回结果
         */
        Map<String, Object> resultMap = new HashMap<>(3);
        resultMap.put("pageDate", pageInfo);
        resultMap.put("invalidProductList", invalidList);
        resultMap.put("promotionsList", totalPromotions);
        // 返回数据
        return ResponseJson.success(resultMap);
    }

    /**
     * 获取协销购物车商品种类
     *
     * @param serviceProviderId 协销Id
     * @param clubId            机构Id
     */
    @Override
    public ResponseJson<Integer> getSellerCartCount(Integer serviceProviderId, Integer clubId) {
        Integer count = cartSellerMapper.getSellerCartCount(serviceProviderId, clubId);
        if (null == count) {
            count = 0;
        }
        return ResponseJson.success(count);
    }

    /**
     * 协销添加购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      clubId       机构ID
     *                      type         商品类型：1单一商品，2组合商品
     *                      productId    单一商品id
     *                      productNum   单一商品数量
     *                      productInfo  组合商品信息：Json字符串格式[{"id":4351,"num":1},{}]
     *                      }
     */
    @Override
    public ResponseJson<Integer> addShoppingCart(SellerCartDto sellerCartDto) {
        if (1 == sellerCartDto.getType()) {
            // 单一商品
            addSellerCart(sellerCartDto);
        } else if (2 == sellerCartDto.getType()) {
            try {
                JSONArray productArr = JSONArray.parseArray(sellerCartDto.getProductInfo());
                for (Iterator iterator = productArr.iterator(); iterator.hasNext();) {
                    JSONObject product = (JSONObject) iterator.next();
                    Integer productId = product.getInteger("id");
                    Integer productNum = product.getInteger("num");
                    sellerCartDto.setProductId(productId);
                    sellerCartDto.setProductNum(productNum);
                    addSellerCart(sellerCartDto);
                }
            } catch (Exception e) {
                log.error("组合商品信息格式错误", e);
                return ResponseJson.error("组合商品信息格式错误！Json字符串格式[{\"id\":4351,\"num\":1},{}]", null);
            }
        }
        // 返回购物车种类数量
        return getSellerCartCount(sellerCartDto.getServiceProviderId(), sellerCartDto.getClubId());
    }

    /**
     * 添加购物车，插入数据库
     * @param sellerCartDto
     */
    private void addSellerCart(SellerCartDto sellerCartDto) {
        SellerCartPo cart = cartSellerMapper.getSellerCart(sellerCartDto);
        if (cart != null) {
            // 购物车已存在该商品，更新数量
            cart.setNum(cart.getNum() + sellerCartDto.getProductNum());
            cart.setAddTime(new Date());
            cartSellerMapper.updateSellerCart(cart);
        } else {
            // 添加新购物车
            cart = new SellerCartPo();
            cart.setServiceProviderId(sellerCartDto.getServiceProviderId());
            cart.setClubId(sellerCartDto.getClubId());
            cart.setProductId(sellerCartDto.getProductId());
            cart.setNum(sellerCartDto.getProductNum());
            cart.setAddTime(new Date());
            cartSellerMapper.insertSellerCart(cart);
        }
    }

    /**
     * 协销更新购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      id           购物车Id
     *                      productNum   商品数量
     *                      }
     */
    @Override
    public ResponseJson<Integer> updateSellerCart(SellerCartDto sellerCartDto) {
        SellerCartPo cart = cartSellerMapper.getSellerCartById(sellerCartDto.getId());
        if (!sellerCartDto.getServiceProviderId().equals(cart.getServiceProviderId())) {
            return ResponseJson.error("协销Id不正确！",null);
        }
        cart.setNum(cart.getNum() + sellerCartDto.getProductNum());
        cart.setAddTime(new Date());
        cartSellerMapper.updateSellerCart(cart);
        return ResponseJson.success(0);
    }

    /**
     * 协销删除购物车
     *
     * @param sellerCartDto {
     *                      serviceProviderId 协销Id
     *                      cartIds           购物车IDs
     *                      }
     */
    @Override
    public ResponseJson<Integer> deleteSellerCart(SellerCartDto sellerCartDto) {
        List<Integer> cartIdList = Lists.newArrayList();
        if (sellerCartDto.getCartIds().contains(",")) {
            String[] productArr = sellerCartDto.getCartIds().split(",");
            for (String id : productArr) {
                cartIdList.add(Integer.parseInt(id));
            }
        } else {
            cartIdList.add(Integer.parseInt(sellerCartDto.getCartIds()));
        }
        cartSellerMapper.deleteSellerCartByIds(sellerCartDto.getServiceProviderId(), cartIdList);
        return ResponseJson.success(0);
    }


}
